@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2021
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,14 +26,14 @@ namespace polymake { namespace matroid {
  * @brief For two lists of subsets of the same ground set, computes the list of pairwise unions that
  * have maximal cardinality.
  */
-Array<Set<int>> basis_union(const Array<Set<int>>& b1, const Array<Set<int>>& b2)
+Array<Set<Int>> basis_union(const Array<Set<Int>>& b1, const Array<Set<Int>>& b2)
 {
-  int max_size = 0;
-  Set<Set<int>> result;
+  Int max_size = 0;
+  Set<Set<Int>> result;
   for (auto i1 = entire(b1); !i1.at_end(); ++i1) {
     for (auto i2 = entire(b2); !i2.at_end(); ++i2) {
-      Set<int> union_set = (*i1) + (*i2);
-      int us_size = union_set.size();
+      Set<Int> union_set = (*i1) + (*i2);
+      Int us_size = union_set.size();
       if (us_size >= max_size) {
         if (us_size > max_size) {
           result.clear();
@@ -42,29 +43,28 @@ Array<Set<int>> basis_union(const Array<Set<int>>& b1, const Array<Set<int>>& b2
       }
     }
   }
-  return Array<Set<int> >(result);
+  return Array<Set<Int>>(result);
 }
 
 // Computes matroid union
-perl::Object matroid_union(const Array<perl::Object>& matroid_list)
+BigObject matroid_union(const Array<BigObject>& matroid_list)
 {
   if (matroid_list.empty())
     throw std::runtime_error("Matroid union: Empty list of matroids");
-  Array<Set<int>> result_bases;
-  const int n = matroid_list[0].give("N_ELEMENTS");
+  Array<Set<Int>> result_bases;
+  const Int n = matroid_list[0].give("N_ELEMENTS");
 
-  for (const perl::Object& m : matroid_list) {
-    const Array<Set<int>> m_bases = m.give("BASES");
+  for (const BigObject& m : matroid_list) {
+    const Array<Set<Int>> m_bases = m.give("BASES");
     if (result_bases.empty())
       result_bases = m_bases;
     else
       result_bases = basis_union(result_bases, m_bases);
   }
 
-  perl::Object result("Matroid");
-  result.take("N_ELEMENTS") << n;
-  result.take("BASES") << result_bases;
-  return result;
+  return BigObject("Matroid",
+                   "N_ELEMENTS", n,
+                   "BASES", result_bases);
 }
 
 UserFunction4perl("# @category Producing a matroid from matroids"
